
//****************************
// Handling the lists of tests
//****************************


// list of tests
var tests = new Array();

// Some constants to make it easier
var NAME         = 0;
var FILE         = 1;
var DESCRIPTION  = 2;
var CATEGORY     = 3;
var CATEGORY_NUM = 4;

function addTest(filename, name, description, category)
{
    var cat = addCategory(category);
    tests[tests.length] = new Array(name, filename, description, cat);
}

//*********************************
// Handling the lists of categories
//
// each test is attached to one category
//
//*********************************


var categories = new Array();
categories[0] = "All test sets";

var category = 0;

function addCategory(name)
{
    var i = 0;
    while (i < categories.length) {
	if (categories[i] == name) return i;
	i++;
    }
    categories[i] = name;
    return i;
}

function switchCategory(cat)
{
    category = cat;

    var tables = document.getElementById("tables");

    for (var c = 1; c < categories.length; c++ ) {
	if (cat == 0) {
	    tables.childNodes.item(c-1).style.cssText = '';
	} else if (c == category) {
	    // setAttribute("class" or "style" doesn't work in IE :-(
	    tables.childNodes.item(c-1).style.cssText = '';
	} else {
	    // setAttribute("class" or "style" doesn't work in IE :-(
	    tables.childNodes.item(c-1).style.cssText = 'display:none';
	}
    }
    
}

//***************************
// Handling the player to use
//***************************

// list of players
var players = new Array();
// the player in use
var player = -1;
var autostart = false;

// A constant to make it easier to access the array
// var NAME = 0; is already defined
var ACTIVATE_FUNCTION_NAME = 1;
var STOP_FUNCTION_NAME = 2;

function addPlayer(name, activateFunctionName, stopFunctionName)
{
    players[players.length] = new Array(name,
					activateFunctionName,
					stopFunctionName);
}


// switch from one player to an other
function switchPlayer(nPlayer) {

    // remove the test currently in use
    removeTest();

    // reset the table of pass/fail
    resetAll();

    // reset the report
    var report_content = document.getElementById("report_content");
    report_content.innerHTML = '';

    // switch
    player = nPlayer;

}

function switchAutostart(nAutostart)
{
    autostart = nAutostart;
}


//***************************
// Handling a test
//***************************

var currentTest = -1;

// Run a dedicated test with the appropriate player
function activeTest(test_number) 
{
    if (test_number > -1 && test_number < tests.length) {
	var title=document.getElementById("title");
	title.innerHTML = tests[test_number][NAME];
	var descr=document.getElementById("description");
	descr.innerHTML = tests[test_number][DESCRIPTION];
	addResultButtons(test_number);

	eval(players[player][ACTIVATE_FUNCTION_NAME] + "("
	     + test_number + ",\""
	     + tests[test_number][FILE] + "\","
	     + autostart
	     + ")");
	currentTest = test_number;
    }
}

// return the next test number to be executed or -1 if none
function nextTestNumber(test_number)
{
    var i = test_number + 1;
    while (i < results.length) {
	if (category == 0 || category == tests[i][CATEGORY]) {
	    return i;
	}
	i++;
    }
    return -1;
}

// stop the current test and move to the next
function moveToNextTest(test_number)
{
    var next = nextTestNumber(test_number);
    if (next == -1) {
	removeTest();
    } else {
	stopTest(currentTest);
	activeTest(next);
    }
}

// stop the player if running
function stopTest(test_number)
{
    eval(players[player][STOP_FUNCTION_NAME] + "("
	     + test_number + ")");
    currentTest = -1;
}

// Remove the current test
// stop the player if running and clear the test area
function removeTest()
{
    // first stop the test
    stopTest(currentTest);

    // clean up the HTML elements
    var div = document.getElementById("testarea");
    div.innerHTML = '';
    var title = document.getElementById("title");
    title.innerHTML = "";
    var descr = document.getElementById("description");
    descr.innerHTML = "";
    var p = document.getElementById("result");
    p.innerHTML = "";
}

//************************************
// Handle the fail/pass/unknown status
//************************************

var results = new Array();

var skipOnResult = false;

// Some constants to make it easier to access the array
var UNKNOWN = 0;
var PASS    = 1;
var FAIL    = 2;
var CANT_TELL = 3;

function initResults()
{
    for (i = 0; i < tests.length; i++) {
	results[i] = UNKNOWN;
    }
}

// reset the table of pass/fail
function addResultButtons(test_number)
{
    var p = document.getElementById("result");
    s =
	"<input value='Pass' type='button' onclick='pass(" + test_number 
	+ ");'> <input type='button' value='Fail' onclick='fail("
	+ test_number + ");'> <input type='button' value='Cannot tell' onclick='cant_tell("
	+ test_number + ");'>";
    if (skipOnResult) {
	s += " <input value='Skip' type='button' onclick='skip(" + test_number 
	    + ");'>";
    }
    p.innerHTML = s;
}

function switchSkipOnResult(nSkip)
{
    skipOnResult = nSkip;
}

function setResultElement(e, result)
{
    var s = '';
    if (result == PASS) {
	s = 'pass';
    } else if (result == FAIL) {
	s = 'fail';
    } else if (result == CANT_TELL) {
	s = "cannot tell";
    }
    if (result != UNKNOWN) {
	e.innerHTML = s;
	e.setAttribute('className', s);    
	e.setAttribute('class', s);    
    }
}

// indicate that the test was passed
function pass(test_number)
{
    var e = document.getElementById("r" + test_number);
    setResultElement(e, PASS);

    results[test_number] = PASS;

    if (skipOnResult) {
	moveToNextTest(test_number);
    }
}

// indicate that the test was failed
function fail(test_number)
{
    var e = document.getElementById("r" + test_number);
    setResultElement(e, FAIL);

    results[test_number] = FAIL;

    if (skipOnResult) {
	moveToNextTest(test_number);
    }
}

// can't tell if the test was passed or failed
function cant_tell(test_number)
{
    var e = document.getElementById("r" + test_number);
    setResultElement(e, CANT_TELL);

    results[test_number] = CANT_TELL;
    if (skipOnResult) {
	moveToNextTest(test_number);
    }
}

// Just skip the test and move to next
function skip(test_number)
{
    moveToNextTest(test_number);
}

// reset the table of pass/fail/can't tell
function resetAll()
{
    for (i = 0; i < tests.length; i++) {
	var e = document.getElementById("r" + i);
	e.innerHTML = '';
	e.removeAttribute('className');
	e.removeAttribute('class');
	// don't forget to clear the results array as well
	results[i] = UNKNOWN;
    }    
}

//*******************************************
// Generate a report of all pass/fail/unknown
//*******************************************

function report()
{
    var pass      = new Array();
    var fail      = new Array();
    var cant_tell = new Array();


    for (i = 0; i < results.length; i++) {
	if (results[i] == PASS) {
	    pass[pass.length] = i;
	} else if (results[i] == FAIL) {
	    fail[fail.length] = i;
	} else if (results[i] == CANT_TELL) {
	    cant_tell[cant_tell.length] = i;
	}
    }
    var total = pass.length+fail.length+cant_tell.length;

    var report_content = document.getElementById("report_content");
    report_content.innerHTML = '';
    var p = document.createElement("p");


    p.innerHTML = new Date().toUTCString();
    report_content.appendChild(p);

    p = document.createElement("p");
    p.innerHTML = "Player: " + players[player][NAME];
    report_content.appendChild(p);
    
    p = document.createElement("p");
    p.innerHTML = total	+ " test result" 
	+ ((total==0)?"":"s")
	+ " collected out of " + tests.length + " tests.";
    report_content.appendChild(p);
    if (total != 0) {
	if (pass.length != 0) {
	    p = document.createElement("p");
	    var s = '';
	    for (i = 0; i<pass.length; i++) {
		if (s != '') {
		    s += ', ';
		}
		s += tests[pass[i]][NAME];
	    }
	    p.innerHTML = pass.length + " test"
		+ ((pass.length>1)?"s":"") + " passed: " + s + ".";
	    report_content.appendChild(p);
	}
	if (fail.length != 0) {
	    p = document.createElement("p");
	    s = '';
	    for (i = 0; i<fail.length; i++) {
		if (s != '') {
		    s += ', ';
		}
		s += tests[fail[i]][NAME];
	    }
	    p.innerHTML = fail.length + " test" 
		+ ((fail.length>1)?"s ":" ") + " failed:" + s + ".";
	    report_content.appendChild(p);
	}
	if (cant_tell.length != 0) {
	    p = document.createElement("p");
	    s = '';
	    for (i = 0; i<cant_tell.length; i++) {
		if (s != '') {
		    s += ', ';
		}
		s += tests[cant_tell[i]][NAME];
	    }
	    p.innerHTML = cant_tell.length + " test" 
		+ ((cant_tell.length>1)?"s ":" ") + " cannot tell:" + s + ".";
	    report_content.appendChild(p);
	}
    }
}

//*************************************************
// handle the selection of a category or the report
//*************************************************

function handleSelection(index)
{
    var r = document.getElementById("report");
    removeTest();
    if (index == categories.length) {
	switchCategory(-1);
	report();
	// setAttribute("class" or "style" doesn't work in IE :-(
	r.style.cssText = '';
    } else {
	// setAttribute("class" or "style" doesn't work in IE :-(
	r.style.cssText = 'display:none';
	switchCategory(index);
    }
}

//********************************************
// init():
// - initialize the results
// - build player list
// - initialize the player
// - build category list
// - build test table
// - initialize skipOnResult and autostart
//********************************************

function init() {
    if (!(document.implementation
	  && document.implementation.hasFeature)) {
	// no DOM support :-(
	return;
    }

    initResults();

    // populate the player list
    var select = document.getElementById("players");

    for (i=0; i < players.length; i++) {
	var opt = document.createElement("option");
	opt.innerHTML = players[i][NAME];
	select.appendChild(opt);
    }
    if (players.length > 0) {
	player = 0;
    }

    // populate the category list
    var select = document.getElementById("categories");

    for (i=0; i < categories.length; i++) {
	var opt = document.createElement("option");
	opt.innerHTML = categories[i];
	select.appendChild(opt);
    }
    var opt = document.createElement("option");
    opt.innerHTML = "See the report";
    select.appendChild(opt);

    var tables = document.getElementById("tables");

    for (c=categories.length-1; c>=1;c--) {
	var div = document.createElement("div");
	div.setAttribute("className", "table");
	div.setAttribute("class", "table");
	div.setAttribute("id", "cat" + c);
	var h3 = document.createElement("h3");
	h3.innerHTML = "Test set: " + categories[c];
	div.appendChild(h3);
	// populate the HTML table with the tests
	var table = document.createElement("table");
	// don't forget to add a tbody element for IE!
	var tbody = document.createElement("tbody");
	table.setAttribute("id", "tcat" + c);
	for (i=0; i < tests.length; i++) {
	    if (c == tests[i][CATEGORY]) {
		var tr = document.createElement("tr");
		var th = document.createElement("th");
		var b = document.createElement("input");
		b.setAttribute("type", "button");
		// b.setAttribute("onclick", "activeTest(i)"); doesn't work on IE :-(
		// so here is a workaround
		b.setAttribute("test", (i).toString(10));
		b.onclick = function () {
		    activeTest(parseInt(this.getAttribute("test")));
		};
		b.setAttribute("value", tests[i][NAME]);
		th.appendChild(b);
		tr.appendChild(th);
		td = document.createElement("td"); // for the source
		var a = document.createElement("a");
		a.setAttribute("href", tests[i][FILE]);
		a.innerHTML = "[source]";
		td.appendChild(a);
		tr.appendChild(td);
		td = document.createElement("td"); // for the result
		td.setAttribute("id", "r" + i);
		tr.appendChild(td);
		tbody.appendChild(tr);
		table.appendChild(tbody);
	    }
	}
	div.appendChild(table);
	tables.insertBefore(div, tables.firstChild);
    }
    handleSelection(0);

    // initialize skipOnResult
    var nSkipOnResult = document.getElementById("skipOnResult").checked;
    switchSkipOnResult(nSkipOnResult);

    // initialize autostart
    var nAutostart = document.getElementById("autostart").checked;
    switchAutostart(nAutostart);
}
